import os


def is_path(path: str) -> bool:
    return os.path.isfile(path)


def image(image_src: str):
    return [
        {
            "type": "appBlock",
            "content": [
                {
                    "type": "imageDisplayBlock",
                    "attrs": {
                        "imageSrc": image_src,
                        "title": "",
                        "width": "100%",
                        "height": "auto",
                    },
                }
            ],
        }
    ]



import uuid

from copy import deepcopy
from typing import Optional, Dict, Any
from abc import ABC


class Tab(ABC):
    BLANK: Dict[str, Any]

    def __init__(self, deployable_id: Optional[str] = None) -> None:

        self._deployable_id = (
            str(uuid.uuid4()) if deployable_id is None else deployable_id
        )

    @property
    def deployable_id(self):
        return self._deployable_id

    @property
    def config(self) -> Dict[str, Any]:
        return self._config

    @config.setter
    def config(self, value: Dict[str, Any]):
        self._config = value

    def json(self):
        return self.config

    def reset(self):
        self.config = deepcopy(self.BLANK)



"""Json Encoder utility

To invoke JSON encoder:

```
    from relevanceai import json_encoder
```

"""
import datetime
import dataclasses
from enum import Enum
from pathlib import PurePath
from types import GeneratorType
import numpy as np
import collections
import pandas as pd

from ipaddress import (
    IPv4Address,
    IPv4Interface,
    IPv4Network,
    IPv6Address,
    IPv6Interface,
    IPv6Network,
)

from enum import Enum
from types import GeneratorType
import datetime
from uuid import UUID
from collections import deque
from pathlib import Path
from typing import Any

# Taken from pydanitc.json
ENCODERS_BY_TYPE = {
    bytes: lambda o: o.decode(),
    datetime.date: lambda o: o.isoformat(),
    datetime.datetime: lambda o: o.isoformat(),
    datetime.time: lambda o: o.isoformat(),
    datetime.timedelta: lambda td: td.total_seconds(),
    Enum: lambda o: o.value,
    frozenset: list,
    deque: list,
    GeneratorType: list,
    IPv4Address: str,
    IPv4Interface: str,
    IPv4Network: str,
    IPv6Address: str,
    IPv6Interface: str,
    IPv6Network: str,
    Path: str,
    set: list,
    UUID: str,
}


def json_encoder(obj: Any, force_string: bool = False):
    """Converts object so it is json serializable
    If you want to add your own mapping,
    customize it this way;

    Parameters
    ------------
    obj: Any
        The object to convert
    force_string: bool
        If True, forces the object to a string representation. Used mainly for
        analytics tracking.

    Example
    --------

    YOu can use our JSON encoder easily.
    >>> documents = [{"value": np.nan}]
    >>> client.json_encoder(documents)

    If you want to use FastAPI's json encoder, do this:
    >>> from fastapi import jsonable_encoder
    >>> client.json_encoder = jsonable_encoder

    """
    # Loop through iterators and convert
    if isinstance(obj, (list, set, frozenset, GeneratorType, tuple, collections.deque)):
        encoded_list = []
        for item in obj:
            encoded_list.append(json_encoder(item, force_string=force_string))
        return encoded_list

    # Loop through dictionaries and convert
    if isinstance(obj, dict):
        encoded_dict = {}
        for key, value in obj.items():
            encoded_key = json_encoder(key, force_string=force_string)
            encoded_value = json_encoder(value, force_string=force_string)
            encoded_dict[encoded_key] = encoded_value
        return encoded_dict

    # Custom conversions
    if dataclasses.is_dataclass(obj):
        return dataclasses.asdict(obj)
    if isinstance(obj, (np.ndarray, np.generic)):
        return json_encoder(obj.tolist(), force_string=force_string)
    if isinstance(obj, pd.DataFrame):
        return json_encoder(obj.to_dict(), force_string=force_string)
    if isinstance(obj, Enum):
        return obj.value
    if isinstance(obj, PurePath):
        return str(obj)
    if isinstance(obj, (str, int, type(None))):
        return obj
    if isinstance(obj, float):
        if pd.isna(obj):
            return None
        else:
            return obj

    if type(obj) in ENCODERS_BY_TYPE:
        return ENCODERS_BY_TYPE[type(obj)](obj)  # type: ignore

    if force_string:
        return repr(obj)

    raise ValueError(f"{obj} ({type(obj)}) cannot be converted to JSON format")


class JSONEncoderUtils:
    def json_encoder(self, obj, force_string: bool = False):
        return json_encoder(obj, force_string=force_string)



from enum import Enum


BASE_URL = "https://api.us-east-1.relevance.ai/latest"

WELCOME_MESSAGE = "Welcome to RelevanceAI AI Dashboard. Logged in as {}."

ONE_MB = 2**20


class Colours(Enum):
    INDIGO = "INDIGO"
    TEAL = "TEAL"
    PINK = "PINK"
    VIOLET = "VIOLET"
    YELLOW = "YELLOW"
    BLUE = "BLUE"



from typing import Optional, Dict, Any

from ai_dashboard.tabs import abstract_tab


class CategoryView(abstract_tab.Tab):
    ID = "CLUSTER_VIEW"

    BLANK: Dict[str, Any] = {
        "activeFilterGroup": "",
        "color": None,
        "configuration": {},
        "name": "",
        "type": ID,
    }

    def __init__(self, config: Optional[Dict[str, Any]] = None):
        super().__init__(endpoints)

        if config is not None:
            self._config = config
        else:
            self.reset()



from typing import Optional, List, Dict, Any


def aggregation_chart(
    groupby: List[Dict[str, Any]],
    metric: List[Dict[str, Any]],
    title: Optional[str] = None,
    page_size: int = 20,
    show_frequencies: bool = False,
    sort_direction: str = "desc",
):
    """
    Example for aggregates
    [
        {
            "agg": "category",
            "field": "_cluster_.desc_all-mpnet-base-v2_vector_.kmeans-8",
            "name": "category _cluster_.desc_all-mpnet-base-v2_vector_.kmeans-8",
            "aggType": "groupby",
        },
        {
            "agg": "avg",
            "field": "_sentiment_.desc.cardiffnlp-twitter-roberta-base-sentiment.overall_sentiment_score",
            "name": "avg desc (Sentiment Score)",
            "aggType": "metric",
        },
    ]
    """
    assert sort_direction in {"Descending", "Ascending"}

    for query in groupby:
        query["aggType"] = "groupby"

    for query in metric:
        query["aggType"] = "metric"

    return [
        {
            "type": "appBlock",
            "content": [
                {
                    "type": "datasetAggregation",
                    "attrs": {
                        "uid": "",
                        "title": title,
                        "chartType": "column",
                        "filters": [],
                        "xAxis": {
                            "fields": [],
                            "numResults": page_size,
                            "resortAlphanumerically": False,
                        },
                        "yAxis": {
                            "fields": [],
                            "showFrequency": show_frequencies,
                            "sortBy": "",
                            "sortDirection": sort_direction,
                        },
                        "timeseries": {
                            "field": "insert_date_",
                            "interval": "monthly",
                        },
                        "sentiment": {
                            "field": "",
                            "mode": "overview",
                            "interval": "monthly",
                        },
                        "wordCloud": {"mode": "cloud"},
                    },
                }
            ],
        },
    ]



import uuid
import numpy as np

from typing import Union, Optional, Dict, Any, List

from ai_dashboard.api import endpoints
from ai_dashboard.tabs import abstract_tab
from ai_dashboard.tabs.data_report import constants
from ai_dashboard.tabs.data_report import aggregation_chart
from ai_dashboard.tabs.data_report import metric
from ai_dashboard.tabs.data_report import markdown
from ai_dashboard.tabs.data_report import image
from ai_dashboard.constants import Colours

import plotly.express as px


class DataReport(abstract_tab.Tab):
    ID = "REPORT"

    BLANK: Dict[str, Any] = {
        "activeFilterGroup": "",
        "color": None,
        "configuration": {
            "content": {
                "type": "doc",
                "content": [],
            }
        },
        "name": "",
        "type": ID,
    }

    def __init__(
        self,
        endpoints: endpoints.Endpoints,
        dataset_id: str,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ):
        super().__init__()

        self._endpoints = endpoints
        self.title = title

        if config is not None:
            self._config = config
        else:
            self.reset()

        self.dataset_id = dataset_id

        if colour is not None:
            colour_values = list(map(lambda x: x._value_, Colours))
            assert colour in colour_values, f"Colour must one of {colour_values}"

        self.config["colour"] = colour
        self.config["configuration"]["title"] = title
        self.config["name"] = name

    @classmethod
    def from_text(
        cls,
        text: str,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
    ):
        if colour is not None:
            assert colour in constants.colours
        data_report = cls(title=title)
        data_report.config = {
            "name": name,
            "type": "REPORT",
            "color": colour.upper() if colour is not None else colour,
            "configuration": {
                "title": title,
                "content": {
                    "type": "doc",
                    "content": [
                        {
                            "type": "appBlock",
                            "content": [
                                {
                                    "type": "paragraph",
                                    "attrs": {"textAlign": "left"},
                                    "content": [
                                        {
                                            "type": "text",
                                            "text": text,
                                        }
                                    ],
                                }
                            ],
                        }
                    ],
                },
            },
            "activeFilterGroup": "",
        }
        return data_report

    @classmethod
    def from_markdown(
        cls,
        text: str,
        title: str,
        name: Optional[str] = None,
        colour: Optional[str] = None,
    ):
        data_report = cls(title=title)
        md = markdown.MarkDown()
        content = md.parse(text)
        data_report.config = {
            "name": name,
            "type": cls.ID,
            "color": colour.upper() if colour is not None else colour,
            "configuration": {
                "title": title,
                "content": {
                    "type": "doc",
                    "content": content,
                },
            },
            "activeFilterGroup": "",
        }
        return data_report

    def insert_local_medias(self, file_paths: List[str]) -> List[str]:
        presigned_urls = self._endpoints._get_file_upload_urls(
            self.dataset_id,
            files=file_paths,
        )
        urls = []
        for index, file_path in enumerate(file_paths):
            url = presigned_urls["files"][index]["url"]
            upload_url = presigned_urls["files"][index]["upload_url"]
            with open(file_path, "rb") as fn_byte:
                media_content = bytes(fn_byte.read())
            urls.append(url)
            response = self._endpoints._upload_media(
                presigned_url=upload_url,
                media_content=media_content,
            )
            assert response.status_code == 200
        return urls

    def add_markdown(self, text: str):
        md = markdown.MarkDown()
        content = md.parse(text)
        config = self.config["configuration"]
        if "content" not in config:
            self.config["configuration"]["content"] = {
                "type": "doc",
                "content": [],
            }
        self.config["configuration"]["content"]["content"] += content

    def add_aggregation(
        self,
        groupby: List[Dict[str, Any]],
        metric: List[Dict[str, Any]],
        title: Optional[str] = None,
        page_size: int = 20,
        show_frequencies: bool = False,
        sort_direction: str = "Descending",
    ):
        chart = aggregation_chart.aggregation_chart(
            groupby=groupby,
            metric=metric,
            title=title,
            page_size=page_size,
            show_frequencies=show_frequencies,
            sort_direction=sort_direction,
        )
        self.config["configuration"]["content"]["content"] += chart

    def add_metric(
        self,
        query: List[Dict[str, Any]],
        title: Optional[str] = None,
        show_frequencies: bool = False,
        sort_direction: str = "Descending",
    ):
        chart = metric.metric(
            query=query,
            title=title,
            show_frequencies=show_frequencies,
            sort_direction=sort_direction,
        )
        self.config["configuration"]["content"]["content"] += chart

    def add_image(self, image_src: Union[bytes, str]):
        if isinstance(image_src, bytes):
            presigned_urls = self._endpoints._get_file_upload_urls(
                self.dataset_id,
                files=[str(uuid.uuid4())],
            )
            url = presigned_urls["files"][0]["url"]
            upload_url = presigned_urls["files"][0]["upload_url"]
            response = self._endpoints._upload_media(
                presigned_url=upload_url,
                media_content=image_src,
            )
            image_src = url
        else:
            image_src = self.insert_local_medias([image_src])[0]
        chart = image.image(image_src)
        self.config["configuration"]["content"]["content"] += chart

    def add_plotly(
        self,
        fig: Any,
        title: str,
        width: int = None,
        height: int = None,
        width_percentage: int = 100,
        options=None,
    ):
        if options is None:
            options = {"displayLogo": False}
        try:
            import plotly
        except ImportError:
            raise ImportError(
                ".plotly requires plotly to be installed, install with 'pip install -U plotly'."
            )
        layout = fig._layout
        if width:
            layout["width"] = width
        else:
            if "width" in layout and layout["width"] == np.inf:
                layout["width"] = "auto"
        if height:
            layout["height"] = height
        else:
            if "height" in layout and layout["height"] == np.inf:
                layout["height"] = "auto"
        chart = {
            "type": "appBlock",
            # "attrs" : {"id": str(uuid.uuid4())},
            "content": [
                {
                    "attrs": {
                        "height": "auto",
                        "data": fig._data,
                        "layout": layout,
                        "options": options,
                        "title": title,
                        "width": f"{width_percentage}%",
                    },
                    "type": "plotlyChart",
                }
            ],
        }
        self.config["configuration"]["content"]["content"] += [chart]



import marko
import uuid

from typing import Any, Dict, List, Union
from marko import block
from marko.block import BlockElement
from marko.inline import InlineElement


class MarkDown:
    def _format_block(self, obj):

        block = dict(
            type=self._get_marko_block_type(obj),
            attrs=dict(id=str(uuid.uuid4())),
        )
        if hasattr(obj, "children"):
            block["content"] = []

            for child in obj.children:
                formatted_child = None

                if isinstance(child, BlockElement):
                    formatted_child = self._format_block(child)

                elif isinstance(child, InlineElement):
                    formatted_child = self._format_inline(child)

                block["content"].append(formatted_child)

        if hasattr(obj, "level"):
            block["attrs"]["level"] = obj.level

        if hasattr(obj, "start"):
            block["attrs"]["start"] = obj.start

        return block

    def _format_inline(self, obj):
        block = dict(
            type="text",
            marks=[],
            attrs=dict(id=str(uuid.uuid4())),
        )
        from marko import inline

        if hasattr(obj, "children"):
            if isinstance(obj.children, list):

                for child in obj.children:
                    if isinstance(child, inline.RawText):
                        block["text"] = child.children

            elif isinstance(obj.children, str):
                block["text"] = obj.children

        if isinstance(obj, inline.StrongEmphasis):
            block["marks"].append(dict(type="bold"))  # type: ignore

        elif isinstance(obj, inline.Emphasis):
            block["marks"].append(dict(type="italic"))  # type: ignore

        return block

    def _get_marko_block_type(self, obj) -> Union[None, str]:
        if isinstance(obj, block.CodeBlock):
            return "paragraph"

        if isinstance(obj, block.BlankLine):
            return "paragraph"

        if isinstance(obj, block.Document):
            return "paragraph"

        if isinstance(obj, block.Heading):
            return "heading"

        if isinstance(obj, block.FencedCode):
            return "test"

        if isinstance(obj, block.List):
            return "orderedList"

        if isinstance(obj, block.ListItem):
            return "listItem"

        if isinstance(obj, block.Quote):
            return "blockquote"

        if isinstance(obj, block.Paragraph):
            return "paragraph"

        return None

    def parse(self, raw: str) -> List[Dict[str, Any]]:
        markdown = marko.parse(raw)
        blocks = []
        for child in markdown.children:
            b = dict(
                type="appBlock",
                attrs=dict(id=str(uuid.uuid4())),
                content=[self._format_block(child)],
            )
            blocks.append(b)

        return blocks
